﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Globalization;

namespace ProbabiltyCalc
{
	public class ProbCalculate
	{
		private Csv m_csv;
		private ProbRequest m_request;

		public ProbCalculate(Csv csv, ProbRequest request)
		{
			m_csv = csv;
			m_request = request;
		}

		public void ProcessToFile(string filename)
		{
			using (StreamWriter sw = File.CreateText(filename))
			{
				foreach(Request request in m_request.Requests)
				{
					IList<IList<string>> filteredTable;
					if (request.Filter != null)
					{
						int filterIndex = m_csv.GetColumnIndex(request.Filter.Key);
						filteredTable = new List<IList<string>>();
						foreach (IList<string> row in m_csv.Table)
						{
							if (row[filterIndex] == request.Filter.Value)
							{
								filteredTable.Add(row);
							}
						}
					}
					else
					{
						filteredTable = m_csv.Table;
					}
					// counter for given table, first key is row, second key is column
					Dictionary<string, IDictionary<string, int>> tableItemsCounter = new Dictionary<string, IDictionary<string, int>>();
					int rightColumnIndex = m_csv.GetColumnIndex(request.Right);
					int leftColumnIndex = m_csv.GetColumnIndex(request.Left);
					sw.Write("\t\t");
					string[] columnValues = m_csv.Table.Select(x => x[rightColumnIndex]).Distinct().ToArray();
					string[] rowValues = m_csv.Table.Select(x => x[leftColumnIndex]).Distinct().ToArray();
					foreach (string rowValue in rowValues)
					{
						Dictionary<string, int> rowDict = new Dictionary<string, int>();
						tableItemsCounter[rowValue] = rowDict; 
						foreach(string columnValue in columnValues)
						{
							rowDict[columnValue] = 0;
						}
					}
					foreach (string columnValue in columnValues)
					{
						sw.Write(columnValue + "\t");
					}
					sw.WriteLine();
					foreach (string rowValue in rowValues)
					{
						sw.Write(rowValue + "\t\t");
						foreach (string columnValue in columnValues)
						{
							int count = (from x in filteredTable
									  where x[rightColumnIndex] == columnValue && x[leftColumnIndex] == rowValue
									  select x).Count();
							tableItemsCounter[rowValue][columnValue] = count;
							sw.Write(count + "\t");
						}
						sw.WriteLine();
					}

					sw.WriteLine();
					foreach (string columnValue in columnValues)
					{
						int sumRows = 0;
						foreach (var val in tableItemsCounter.Values)
						{
							sumRows += val[columnValue];
						}
						foreach (string rowValue in rowValues)
						{
							sw.WriteLine(string.Format(CultureInfo.CurrentUICulture, "{0}|{1} = {2:0.00}", rowValue, columnValue, 
								sumRows == 0 ? 0 : tableItemsCounter[rowValue][columnValue] / (double)sumRows));
						}
						sw.WriteLine();
					}
					sw.WriteLine("----------------------");
				}
			}
		}
	}
}
